/**
 * Run nodeunit test.
 * @memberof module:apeman-dev-commons-testing/dev
 * @function runNodeunit
 * @param {string|string[]} pattern - File names
 * @param {object} [options] - Optional settings.
 * @param {function} [callback] - Callback when done.
 */

"use strict";

var argx = require('argx'),
    glob = require('glob'),
    async = require('async');

/** @lends runNodeunit */
function runNodeunit(pattern, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object');
    pattern = args.remain().reduce(function (a, b) {
        return a.concat(b);
    }, []);

    var nodeunit = require('nodeunit');

    var hasReporters = Object.keys(nodeunit.reporters).length > 0;
    if (!hasReporters) {
        callback(null);
        return;
    }
    var reporterName = (options.reporter || 'default'),
        reporter = nodeunit.reporters[reporterName];
    if (!reporter) {
        callback(new Error('Unknown reporter: ' + reporterName));
        return;
    }

    async.waterfall([
        function (callback) {
            _concatPatterns(pattern, callback);
        },
        function (filenames, callback) {
            var options = require('./_nodeunit_options.json');
            reporter.run(filenames, options, callback);
        }
    ], function (err) {
        _done(err);
        callback(err);
    });

}
function _done(err) {
    if (err) {
        console.error(err);
        process.nextTick(function () {
            process.exit(1);
        });
    }
}
function _concatPatterns(patterns, callback) {
    async.concat([].concat(patterns || []), glob, callback);
}


module.exports = runNodeunit;